; *************************************
;
; Direct keyboard input
;
; *************************************

; -------------------------------------
; WaitChr - wait for char from kbd
;
; The char is not printed to screen.
;
; affects: A, X, Y
; return: char in A
; -------------------------------------
WaitChr jsr GetChr  ; char from kbd
        cmp #$00    ; no char?
        beq WaitChr ; wait..
	rts

; -------------------------------------
; XkbdR - reads string from kbd 
;
; string is stored to XkbdBuf
; string is zero terminated
;
; affects: A, X, Y
; uses: xVector
; return: X length of string
;	  zero-flag: string empty
; -------------------------------------
XkbdR   ldx #$00        
XkbdR1  stx xVector ; save X
	jsr WaitChr
        ldx xVector ; restore X
        cmp #$0d    ; RETURN?
        bne XkbdR2
	; -- input finished ---
	lda #$00    ; append zero
        sta XkbdBuf,x
        txa
        pha
        jsr CR
        jsr LF
        pla
        tax
        rts
        
XkbdR2  cmp #$14    ; DEL?
        bne XkbdR3
        ; -- delete --
        cpx #$00    ; at the beginning?
        beq XkbdR1  ; -> ignore
        dex
	txa
	pha
	jsr DEL
	pla
	tax
        jmp XkbdR1  ; next char

XkbdR3  ; -- check range--        
        cmp #$20    ; lower control
        bcc XkbdR1  ; -> ignore
        cmp #$80    ; printable
        bcc XkbdR5  ; -> valid
        cmp #$a0    ; upper control
        bcc XkbdR1  ; -> ignore                
XkbdR5 	; -- store char --
	sta XkbdBuf,x ; to kbd buffer
        inx
	bne XkbdR6  ; buffer full?
	dex	    ; -> remove char
	lda #RBuzz  ; -> buzzer request
	sta BellR
        jmp XkbdR1  ; -> read next

XkbdR6  jsr CPrnChr ; print to screen
        jmp XkbdR1  ; read next


; -------------------------------------
; XkbdNumNL - read number from kbd
; 
; at the end go to new line (CR LF)

; affects: A, X, Y
; uses: yVector
; return: number in A
;         c = 1 valid
;         c = 0 invalid
; -------------------------------------
XkbdNumNL
	jsr XkbdNum
	pha	    ; save char
	rol	    ; save carry
	pha
	jsr CR
	jsr LF
	pla	    ; restore carry
	ror
	pla	    ; restore char
	rts

; -------------------------------------
; XkbdNum - read number from kbd
; 
; affects: A, X, Y
; uses: yVector
; return: number in A
;         c = 1 valid
;         c = 0 invalid
; -------------------------------------
XkbdNum	
	lda #$00
	sta yVector   ; init value
	lda #03
	sta yVector+1 ; init digit count
XkbN1	; --- get digit from kbd ---
	jsr WaitChr
        cmp #$0d    ; RETURN?
	beq XkbEnd  ; -> ready
	jsr TestD
	bcc XkbNerr
	tay	      ; digit to Y
	jsr CPrnChr
	; --- add new digit ---
        lda yVector
	jsr addDecDig
	bcs XkbNerr
        sta yVector
	; --- three digits? ---
	dec yVector+1
	bne XkbN1
XkbEnd	; --- successfull end ---
	lda yVector
	sec
	rts

XkbNerr ; --- unsuccessful end ---
	lda #RBuzz    ; buzzer request
	sta BellR
	clc
	rts


; vim: set syntax=asm6502 :
