; *************************************
; *
; * Zeropage 
; *
; *************************************

; -------------------------------------
; Zeropage CaTer
;
; Zeropage from $02 to 8f is normally 
; used by basic
; -------------------------------------

; --- dump mode ---
; $00 = normal, $01 = dump mode
DMode = $02

; --- esc mode ---
; $00 = normal
; $0f = esc mode
; $ff = esc [ mode
; $f0 = ignore one char
EMode = $03

; --- baud rate ---
; $06 = 300
; $08 = 1200
; $0a = 2400
; $0c = 9600
; $0e = use serial IEC-bus
Baud = $04

; --- Bell request ---
; $00  no request
; $81  bell
; $80  buzzer
; positive value is counted down
; as tone length
BellR = $05

; --- Vector ---
; four vectors in zeropage for
; temporary use.
zVector = $06
yVector = $08
xVector = $0a
wVector = $0c

; --- esc buffer length ---
; points on first free position
EBufL = $0e

; --- esc parameter ---
; numeric parameter in esc sequence
EPar = $0f

; --- scroll region ---
SRS = $10	; first line number
SRE = $11	; last line number

; --- ANSI font attributes ---
; contains three bits
; bit 0 = reverse
; bit 1 = bold
; bit 2 = underline
; bit 6 = background set
; bit 7 = foreground set
Font = $12
; foreground
AFore = $13
; background
ABack = $14

; --- crsr save area ---
; here is crsr info saved with ESC 7
; and restored from with ESC 8
SaveF = $15	; font
SaveR = $16	; reverse mode
SaveRow = $17	; row
SaveCol = $18	; column

; --- Number panel ---
NumMode = $19   ; number mode, $00 = off
NumSize = $1a	; number of chars in buf
NumVal = $1b	; Value in Buffer
NumClose = $1c  ; timeout number panel
		; a value != $00 is 
		; counted down

; --- Xmodem sender ---
; $ff says sender on
; $00 says sender off
Xmodem = $1d 

; --- vector for PrnScr ---
; $1e, $1f
vVector = $1e

; --- Linebreak pending ---
; 0 = not pending, ff = pending
lbPending = $20

; --- crsr invisible ---
; 0 = visible, !0 = invisible
civis = $21

; --- save crsr color for XCon/XCoff --
XCrsr = $22 ; safe crsr colour

; --- Timeout counter for XgetRS ---
Xtime = $23

; --- buffer for IntStr, 6 byte ---
; $24 - $29
DecNum  = $24

; --- buffer for addDecDig ---
mul10buf = $2a

; --- floppy device number ---
Floppy = $2b

; --- unused ---
; $2c - $8a

; --- up9600 ---
; $8a - $8f

; -------------------------------------
; Zeropage Kernal
;
; Zeropage from $90 to $ff is used
; by kernal. 
; Following are the variables accessed
; by CaTer.
; -------------------------------------

; --- kernal status byte ST ---
; error code from kernal routines
; for floppy and tape
ST = $90

; --- kernal clock ---
; count IRQs (1/60 s) in 24 bit int
; lo byte used as 'jiffies' in up9600
ClockH = $a0 ; hi byte = 1.777 min
ClockM = $a1 ; mi byte = 4.266 s
ClockL = $a2 ; lo byte = 1/60 s

; --- reverse mode ---
; 0 = normal, !0 = revers
Rvs = $c7

; --- crsr flag, 0 = on ---
sCrsrOn = $cc    

; --- crsr blink counter ---
sCrsrCnt = $cd    

; --- char under crsr ---
sCrsrChar = $ce    

; --- crsr blink phase ---
; $00 = normal, $01 = revers
sCrsrPhase = $cf    

; --- ptr to start of screen line ---
sLinePtr = $d1 

; --- crsr column ---
sCol = $d3 

; --- crsr row ---
sRow = $d6 

; --- ptr to start of colour line ---
sLineColPtr = $f3 

; *************************************
; *
; * Memory Map
; *
; *************************************

; -------------------------------------
; 6502/6510 stack
;
; $0100 - $01ff
; -------------------------------------

; -------------------------------------
; page $02, $03: BASIC and KERNAL 
;
; Following are the variables accessed
; by CaTer.
; -------------------------------------

; --- colour for writing ---
sColor = $0286 

; --- colour under crsr ---
sCrsrCol = $0287  

; --- CTRL, SHIFT, C= ---
; flags indicating pressed
; contol keys
; 1 SHIFT, 2 C=, 4 CTRL
ControlFlags = $028d

; --- flag for SHIFT + C= ---
; $80 = lock, no charset switch
; $00 = free
ShiftCFlag = $0291

; --- IRQ vector ---
IRQ_Vector = $0314

; -------------------------------------
; screen
;
; $0400 - $07ff
; 1000 char + 8 sprite ptr at end
; -------------------------------------
Screen = $0400
SpritePtr0 = $07f8 ; place 100
SpritePtr1 = $07f9 ; place 10
SpritePtr2 = $07fa ; place 1
SpritePtr3 = $07fb ; background

; -------------------------------------
; basic memory
; 
; $0800 - $9fff
; -------------------------------------

; --- second screen, e. g. help --- 
; not implemented
HelpS = $0800 ; $0800 - $0bff

; --- esc-Buf --- 
EBuf = $0c00 ; $0c00 - $0cff

; --- up9600 ---
; $0d00 - $0fff

; $1000 - $1fff
; VIC sees char-ROM

; --- xmodem ---  
; $1000 - $108f

; --- free ---
; $1090 - $10fd

; --- IEC connection ---
IECdevice = $10fe
IECstrLen = $10ff
IECstring = $1100 ; $1100 - $11ff

; --- buffer for kbd input ---
XkbdBuf = $1200 ; $1200 - $12ff

; --- char ROM --- 
Char = $2000 ; $2000 - $27ff

; --- sprites ---
; $2800 - $2aff
SpriteBase = $2800
SpriteBaseVIC = $a0 ; = SpriteBase / 64

; vim: set syntax=asm6502 :
