; *************************************
; *                                   *
; * XMODEM file transfer protocol     *
; *                                   *
; * This file is part of CaTer.       *
; * (C) Lars Stollenwerk 2001 - 2003  *
; *                                   *
; * CaTer is provided under the terms *
; * of GNU General Public License.    *
; *                                   *
; * For more information see the      *
; * README file.                      *
; *                                   *
; *************************************
 
SOH=1     ; $01 ^A start of header
EOT=4     ; $04 ^D end of transmission
ACK=6     ; $06 ^F Acknowledge (pos.)
NAK=21    ; $15 ^U Negative Acknowledge
CAN=24    ; $16 ^X Cancel

; --- from zeropage ---
; $ff says sender on
; $00 says sender off
;Xmodem = $1c

; --- buffer for XMODEM block ---
; $1000 - $107f
XBuffer = $1000 

XCount  = $1080 ; block counter
XSum    = $1081 ; check sum
Xretry  = $1082 ; counter get/send NAK
Xdouble = $1083 ; flag: dublicate block
Xhecnt  = $1084 ; Header error count
; $1085 free
; $1086 free
; $1087 free
; $1088 free
; $1089 free
; $108a free
; $108b free
; $108c free
; $108d free
; $108e free
; $108f free

; *************************************
;         public routines
; *************************************

; Xrec      XMODEM Receiver
; Xsend     activate sender
; XsendNAK  start sender

; -------------------------------------
; subroutine send answer
;
; answer in A

sAnswer pha         
        jsr GetChr  ; keypress?
        cmp #$03    ; RUN/STOP?
        bne sAnswer1; no -> go on
        
        pla         ; throw away answer
        pla         ; and jsr adress
        pla
        jmp XusrC   ; abort
                
sAnswer1
        pla         ; send answer
        jsr putRS
        rts

; -------------------------------------
; subroutine unsuccessful end
;

XendR   ; --- successful end ---
        ; send ACK, inform user, 
        ; close floppy, end xmodem
        
        lda #ACK    ; quit EOT with ACK
        jsr putRS
	jmp Xend

; -------------------------------------
; subroutine unsuccessful end
;

Xcan    ; --- error - cancel transfer -
        ; send 4 CAN, inform user, 
        ; close floppy, end xmodem
        
        ldy #$04    ; send 4 CAN bytes
Xcan1   lda #CAN
        jsr putRS
        dey
        bne Xcan1
        lda #$63    ; print C
        jmp Xend1                

; -------------------------------------
; receive Xmodem file
;
; -------------------------------------

Xrec    
        lda Xmodem ;<> $00 -> sender on
        beq Xrec1  
        rts        ; exit
        
Xrec1   jsr XCon    ; yellow crsr
	; -- ask user --
        ldx #<XrecvMsg
        ldy #>XrecvMsg
        jsr CPrnStrNL
	; -- open file --
        lda #$57    ; w: open file..
        jsr Xopen   ; .. to write
	; -- inform user --
        ldx #<XtransmissionMsg
        ldy #>XtransmissionMsg
        jsr CPrnStrNL
	; -- init transmission --
        ldx #$01    ; first block is 1
        stx XCount
        dex         ; mark first block..
        stx Xdouble ; ..as no dublicate
        
        ; --- request one block ---
        ; send NAK untill answer 
        
Xreq    lda #$05    ; max 5 tries to..
        sta Xretry  ; ..get block
Xreq1   lda #NAK    ; send NAK
        jsr sAnswer 
        jsr XgetRS  ; read byte
        bcc XrdH    ; got Data?->rdHead
        dec Xretry  ; count down
        bne Xreq1   ; send NAK again        
        jmp Xtimeo  ; max tries>timeout


Xherr   ; --- header error 
	; inform user
	; read until no data follows
	; re-request last block
	; on 3rd try: 
        ; send 4 CAN, inform user, 
        ; close floppy, end xmodem

        lda #$68    ; print H
        jsr CPrnChr ; to screen
	inc Xhecnt  ; header error + 1
	lda #$03    ; 3rd try?
	cmp Xhecnt
    	beq Xcan    ; -> abort

        ; wait for timeout
Xherr1	jsr XgetRS  ; read byte
        bcs Xreq    ; timeout->req block
        jsr GetChr  ; keypress?
        cmp #$03    ; RUN/STOP?
        bne Xherr1  ; no -> wait on
        jmp XusrC   ; yes -> abort
        
XrdH    ; --- read block header ---
        cpy #EOT    ; file end?
        beq XendR
        cpy #CAN    ; cancel?
        beq Xcan
        cpy #SOH    ; begin block?
        bne Xherr   ; no -> Head error
        jsr XgetRS  ; read block #
        bcs Xtimeo  ; timeout?
        cpy XCount
        bne Xherr
        jsr XgetRS  ; read inv. block #
        bcs Xtimeo  ; timeout?
        tya         ; inv.Blk to A
        eor #$ff
        cmp XCount
        bne Xherr
	lda #$00    ; Header Error = 0
	sta Xhecnt
        
        ; -- read 128 bytes data --
        ldx #$00
        stx XSum
XrdB1   txa
        pha        
        jsr XgetRS  ; read data        
        pla 
        tax
        bcs Xtimeo  ; timeout?
        tya         ; data to A
        sta XBuffer,x
        clc         ; add to checksum
        adc XSum
        sta XSum
        inx
        bpl XrdB1
        jsr XgetRS  ; read checksum
        bcs Xtimeo  ; timeout?
        cpy XSum
        beq XrdBok
        
        ; -- checksum wrong --
        lda #$52    ; print r
        jsr CPrnChr
        jmp Xreq    ; read block again
        
XrdBok  ; -- checksum OK --
        lda Xdouble ; dublicate?
        bne XrdBok1
        jsr Xsave   ; no -> save block
XrdBok1 lda #$2e    ; print dot
        jsr CPrnChr
        jsr GetChr  ; keypress?
        cmp #$03    ; RUN/STOP?
        beq XusrC
        lda #ACK    ; requ. next block
        jsr putRS
        jsr XgetRS  ; read byte
        bcc XrdBok2 ; got answer->go on
        
        ; - no answer -
        lda #$72    ; print R
        jsr CPrnChr
        lda #$01    ; mark next block..
        sta Xdouble ; ..as dublicate
        jmp Xreq    ; request

XrdBok2 ; - got answer -
        inc XCount
        lda #$00    ; mark next block..
        sta Xdouble ; ..as first send
        jmp XrdH

; -------------------------------------
; endings for xrec and xsend
;
; -------------------------------------

Xtimeo  ; --- timeout waiting for data 
        ; send 4 CAN, inform user, 
        ; close floppy, end xmodem
        
        lda #$74    ; print T
        jsr CPrnChr ; to screen
        jmp Xcan

Xend    ; --- successful end ---
        ; inform user, 
        ; close floppy, end xmodem
        
        lda #$2a    ; print *
Xend1   jsr CPrnChr ; to screen
        jsr CR      ; next screen line
        jsr LF
        jsr Xclose  ; close floppy
        jmp XCoff   ; normal crsr

XusrC   ; --- user cancel 
        ; send 4 CAN, inform user, 
        ; close floppy, end xmodem

        lda #$75    ; print U
        jsr CPrnChr ; to screen
        jmp Xcan

; -------------------------------------
; Xsend - send xmodem file
;
; -------------------------------------
; -------------------------------------
; C=S jumps to this routine. It toggles
; Xmodem flag, handles floppy file,
; and prints user info.

Xsend   
        jsr XCon    ; yellow crsr
        lda Xmodem  ; sender on?
        beq Xsend0  ; no -> init
        
        ; abort sender
        
        inc Xmodem  ; $ff -> $00
        jsr Xclose  ; close floppy
        ldx #<XabortMsg
        ldy #>XabortMsg
        jmp Xsend00
        
Xsend0  ; init sender and return to
        ; terminal
        
        dec Xmodem  ; $00 -> $ff
        lda #$52    ; r: open file..
        jsr Xopen   ; .. to read
        lda #$00    ; init block count
        sta XCount
        ldx #<XsendMsg
        ldy #>XsendMsg
Xsend00        
        jsr CPrnStrNL
        jsr XCoff   ; yellow off
        rts
        
; -------------------------------------
; If sender is on (Xmodem = $ff) and
; main loop gets NAK, it jsr's here 
; and sending starts

XsendNAK
        ; Xmodem flag for mainloop off
        inc Xmodem    ; $ff -> $00        
        jsr XCon      ; yellow crsr
        ldx #<XtransmissionMsg
        ldy #>XtransmissionMsg
        jsr CPrnStrNL
        
XsdB    ; --- send one block ---
        inc XCount    ; inc block no
        jsr Xload     ; more valid..
        bcc XsdBa     ; ..data -> go on
        ; -- send EOT and get AKN --
        lda #EOT      ; send EOT
        jsr putRS
        jsr XgetRS    ; get answer
        bcs Xend      ; no answer > end
        cpy #NAK      ; repeat req?
        bne XsdB0     ; no->skip
        lda #EOT      ; send EOT
        jsr putRS
        jmp Xend
XsdB0   cpy #ACK      ; right answer?
        beq Xend      ; yes -> end
        lda #$75      ; no -> print ?
        jsr CPrnChr   ; to screen
        jmp Xend      ; end
XsdBa   ; -- send block again --
        jsr GetChr    ; keypress?
        cmp #$03      ; RUN/STOP?
        bne XsdBa0    ; no -> go on
        jmp XusrC     ; yes -> abort
XsdBa0  lda #SOH      ; send SOH
        jsr putRS
        lda XCount    ; send blk no
        jsr putRS
        lda XCount    ; send inv blk no
        eor #$ff
        jsr putRS
        ; -- send 128 bytes data --
        ldy #$00
        sty XSum
XsdB1   lda XBuffer,y ; read buffer
        jsr putRS     ; send data
        clc
        lda XBuffer,y ; add to..
        adc XSum      ; ..checksum
        sta XSum
        iny
        bpl XsdB1     ; next byte
        lda XSum      ; send checksum
        jsr putRS
        ; -- evaluate answer --
        jsr XgetRS    ; get answer
        bcs XwNak     ; no a.->wait NAK
        cpy #ACK      ; ACK?
        bne XsdB2     ; no -> branche
        lda #$2e      ; print dot
        jsr CPrnChr
        jmp XsdB      ; send next block        
XsdB2   cpy #NAK      ; NAK?
        bne XsdB3     ; no -> branche
XrqBa   lda #$52      ; print r
        jsr CPrnChr
        jmp XsdBa     ; send blk again
XsdB3   cpy #CAN      ; CAN?
        bne XwNak     ; no -> branche
        jmp Xcan
XwNak   ; -- wait for NAK --
        lda #$05      ; max 5 tries to..
        sta Xretry    ; ..get answer
XwNak0  dec Xretry    ; count try
        bne XwNak1    ; timeout?
        jmp Xtimeo
XwNak1  jsr GetChr    ; keypress?
        cmp #$03      ; RUN/STOP?
        bne XwNak2    ; no -> go on
        jmp XusrC     ; yes -> abort
XwNak2  jsr XgetRS    ; get answer
        bcs XwNak0    ; no a.->wait NAK
        cpy #NAK      ; got NAK?
        bne XwNak0    ; no -> wait NAK
        jmp XrqBa     ; blk req'ed again

XrecvMsg
	.asc "xmodem rECEIVE - eNTER FILE NAME:"
	.byt $00
XsendMsg  
        .asc "sTART xmodem RECEIVER OR ABORT WITH c=s"
        .byt $00
XabortMsg
        .asc "xmodem SENDER ABORTED"
        .byt $00          
XtransmissionMsg
	.asc "tRANSMISSION START - ABORT WITH rUN/sTOP"
        .byt $00          

; *************************************
;
;          floppy routines
;
; *************************************

; -------------------------------------
; Xopen - opens file 
;
; params: A = 'r' for read 
;         A = 'w' for write
;
; lfn for data: 2
; lfn for error chan: 15
;
; filename is read from prompt
; -------------------------------------
Xopen   pha         ; r/w to stack
        jsr XUPoff
        ; -- open error channel --
        lda #$0f    ; file no f
        ldx Floppy
        ldy #$0f    ; error channel
        jsr SETLFS
        lda #$00    ; no filename
        jsr SETNAM
        jsr OPEN
        ; -- open data channel --
        lda #$02    ; file no 2
        ldx Floppy
        ldy #$02    ; data channel
        jsr SETLFS
        ; -- read file name --
        jsr XkbdR   ; read filename
        lda #$2c    ; append ,
        sta XkbdBuf,x
        inx
        ; -- read file type --
        txa         ; save x
        pha
        ldx #<SXopen ; ask for
        ldy #>SXopen ; filetype
        jsr CPrnStr
        jsr WaitChr ; wait for char ..
        pha         ; save char
        jsr CPrnChr ; print char
        jsr CR
        jsr LF
        pla         ; restore char..
        tay         ; -- to y
        pla         ; restore x
        tax
        tya         ; restore char
        sta XkbdBuf,x ; append filetype
        inx
        lda #$2c    ; append ,
        sta XkbdBuf,x
        inx
        ; -- data direction r/w --
        pla         ; append r/w
        sta XkbdBuf,x
        inx
        ; -- open cmd to floppy --
        txa
        ldx #<XkbdBuf
        ldy #>XkbdBuf
        jsr SETNAM
        jsr OPEN
        jsr XFstate

        jsr XUPon
        rts

SXopen
.asc "fILE tYPE pRG, rEL, sEQ, uSR? "
.byt $00


; -------------------------------------
; Xclose - closes file 
;
; closes lfn  2 for data
; closes lfn 15 for error chan
; -------------------------------------
Xclose  jsr XUPoff

        lda #$02    ; close lfn 2
        jsr CLOSE
        jsr XFstate
        lda #$0f    ; close lfn 15
        jsr CLOSE                

        jsr XUPon
        rts

; -------------------------------------
; Xsave - saves Xmodem block
;
; preparation: Xopen
; affects: A, X, Y
; -------------------------------------
Xsave   jsr XUPoff

        ldx #$02    ; 2=lfn floppy data
        jsr CHKOUT
                
        ldy #$00    ; write buffer
Xsave1  lda XBuffer,y
        jsr CHROUT
        iny        
        bpl Xsave1

        jsr CLRCHN
        jsr XUPon
	rts

; -------------------------------------
; Xload - load a XMODEM block
;
; preparation: Xopen
; affects: A, X, Y
; returns: c=0 if more data found
;          c=1 on file end
; -------------------------------------
Xload
        jsr XUPoff
        ldx #$02     ; 2=lfn floppy data
        jsr CHKIN

        ldy #$00
Xload1  jsr CHRIN     ; read data
        ldx ST        ; end of file?
        bne Xload2    ; -> branche
        sta XBuffer,y ; store data
        iny        
        bpl Xload1

        jmp XloadE  ; return valid blk

Xload2  ; --- file end ---
        cpy #$00    ; any bytes in blk?
        bne XloadF  ; -> fill block

	; retrun invalid data in block
        jsr CLRCHN
        jsr XUPon
        sec         ; block not valid
        rts

XloadF  ; --- fill block with ^Z ---        
        lda #$1a
Xload3  sta XBuffer,y ; store ^Z
        iny        
        bpl Xload3

XloadE  ; return valid data in block
        jsr CLRCHN
        jsr XUPon
        clc         ; block valid
        rts        

; -------------------------------------
; XFstate - read error channel
;
; condition: must be in UPoff ... UPon 
;            lfn 15 is error chan
;
; prints error message to screen
; -------------------------------------
XFstate        
        ; -- read error chan --
        ldx #$0f    ; 15 = lfn err chan
        jsr CHKIN   ; floppy  

XF0     jsr CHRIN   ; get char
        cmp #$0d    ; end of str?
        beq XF1     ; yes -> branche
        jsr CPrnChr ; print char
        jmp XF0     ; next char

XF1     jsr CLRCHN
	jsr CR      ; next screen line
        jsr LF
	rts

; -------------------------------------
; Xcommand - send floppy command
;
; uses: xVector
;
; opens and closes lfn 15 for error chan
; 
; Reads command from keyboard.
; 
; If called whlie xmodem sender is on
; (i.e. waiting for transmissioin to 
; start), return without operation.
; Otherwise, too many files might be 
; opened (max is 3).
; -------------------------------------
Xcommand
	; -- if XMODEM sender on: skip 
	lda Xmodem  ; != 0 -> sender on
	bne Xcom5

        jsr XCon
        jsr XUPoff
	; -- ask for command ---
        ldx #<XcomStr
        ldy #>XcomStr
        jsr CPrnStrNL
        jsr XkbdR   ; read command
	beq Xcom4   ; empty -> abort
        stx xVector ; save cmd length
        ; -- open error channel --
        lda #$0f    ; file no f
        ldx Floppy
        ldy #$0f    ; error channel
        jsr SETLFS
        lda #$00    ; no filename
        jsr SETNAM
        jsr OPEN
        ; -- send command --
        ldx #$0f    ; 15=lfn error chan
        jsr CHKOUT
        ldy #$00
Xcom3   lda XkbdBuf,y
        jsr CHROUT  ; send char
        iny         ; next char
        cpy xVector ; last char?
        bne Xcom3
        lda #$0d    ; send CR
        jsr CHROUT        
	jsr CLRCHN
        ; -- read error ---
        jsr XFstate
        ; -- close error chan --
        lda #$0f    ; close lfn f
        jsr CLOSE                

Xcom4   jsr XUPon
        jsr XCoff
Xcom5   rts

XcomStr	.asc "eNTER FLOPPY COMMAND:"
	.byt $00

; -------------------------------------
; XDir - reads directory and prints to screen
;
; opens and closes lfn 3 for load 
;
; If called whlie xmodem sender is on
; (i.e. waiting for transmissioin to 
; start), return without operation.
; Otherwise, too many files might be 
; opened (max is 3).
; -------------------------------------
XDir    
	; -- if XMODEM sender on: skip 
	lda Xmodem  ; != 0 -> sender on
	bne XDir0

        jsr XCon
        jsr XUPoff
        
        lda #$03    ; file no 3
        ldx Floppy
        ldy #$00    ; load channel
        jsr SETLFS        
        lda #$01    ; one char
        ldx #<XDirStr
        ldy #>XDirStr
        jsr SETNAM
        jsr OPEN
        ldx #$03    ; 3=lfn floppy
        jsr CHKIN

        jsr CHRIN   ; PRG start address
        jsr CHRIN
XDir1   jsr GetChr  ; keypress?
        cmp #$03    ; RUN/STOP?
	beq XDirE   ; -> abort
	jsr CHRIN   ; start next line
        jsr CHRIN
        jsr CHRIN   ; file size
        sta xVector
        jsr CHRIN
        sta xVector+1
        lda ST      ; end of dir?
        bne XDirE
        jsr IntStr  ; file size2string
        ldx #<DecNum
        ldy #>DecNum
        jsr CPrnStr ; print file size
        lda #$20    ; print space
        jsr CPrnChr 
        ; -- print file name --
XDir5   jsr CHRIN
        cmp #$00    ; $00 -> line end
        beq XDir6
        jsr CPrnChr ; print char
        jmp XDir5
        ; -- next file --
XDir6   jsr CR      ; next screen line
        jsr LF
        jmp XDir1   ; next entry
                        
XDirE   ; -- close channel --
	jsr CLRCHN
        lda #$03    ; close lfn 3
        jsr CLOSE                

        jsr XUPon
        jsr XCoff
XDir0   rts
        
XDirStr .asc "$"

; vim: set syntax=asm6502 :
